use anyhow::{bail, Result};
use scraper;

pub fn find_next_chapter(html: &String) -> Result<String> {
    let document = scraper::Html::parse_document(html);
    // let a_selector = scraper::Selector::parse("div>a").unwrap();
    // This one works:  (starts with "/bible/").
    // let a_selector = scraper::Selector::parse(r#"div>a[href ^= "/bible/"]"#).unwrap();
    // This one fails to find what we're looking for.  Contains /bible/ or "bible" both fail.
    // Ok, using href~="/bible/" fails.  It looks for the WORD "/bible/".
    // Using *= finds it anywhere.  Ah HA!
    let a_selector = scraper::Selector::parse(r#"div>a[href*="/bible/"]"#).unwrap();

    for a in document.select(&a_selector) {
        if a.attr("class").is_some() {
            continue;
        }

        // Since the selector finds href containing /bible/, I don't need some of these tests now.
        // I still need this one, so I have the href value.

        if let Some(href) = a.attr("href") {
            //    if href.contains("/bible/") {

            let text = a
                .text()
                .map(|s| {
                    s.trim_matches(char::is_whitespace) // &[' ', '\n', '\t'])
                })
                .filter(|x| {
                    !x.is_empty()
                    // x.chars().any(|c| (c != ' ') && (c != '\n'))
                })
                .collect::<Vec<_>>();
            // println!("TEXT: {:?}", text);

            if text.len() != 1 {
                continue;
            }
            if text[0] != "Next Chapter" {
                // println!("Found: [{:?}]", text[0]);
                continue;
            }
            return Ok(href.to_string());

            //    } else {
            //        println!("href contains: [{}]", href);
            //    }
        }
    }
    bail!("Next Chapter not found.");
}

#[cfg(test)]
mod tests {
    use super::*;

    /// Test HTML as given to us by the website.
    #[test]
    fn chapter_test() {
        let html = String::from(r#"<div class="[pointer-events:all]"><a href="/bible/59/GEN.2.ESV"><div class="flex items-center justify-center bg-white z-[5] h-[50px] w-[50px] rounded-full border-gray-15 border-small border-solid shadow-light-1 active:shadow-dark-1 active:bg-gray-5 active:border-b-gray-5"><svg width="25" height="25" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg" aria-labelledby="Next Chapter" class="text-gray-25"><title id="Next Chapter">Next Chapter</title><path fill-rule="evenodd" clip-rule="evenodd" d="M8.293 18.707a1 1 0 0 1 0-1.414l4.94-4.94a.5.5 0 0 0 0-.707l-4.94-4.939a1 1 0 0 1 1.414-1.414l5.647 5.646a1.5 1.5 0 0 1 0 2.122l-5.647 5.646a1 1 0 0 1-1.414 0Z" fill="currentColor"></path></svg></div></a></div>"#);
        let r = find_next_chapter(&html);
        if !r.is_ok() {
            println!("DEBUG result = {:?}", r);
        }
        assert!(r.is_ok());
        let link = r.unwrap();
        assert_eq!(link, "/bible/59/GEN.2.ESV");
    }

    /// This tests when the HTML has been tidied.
    /// 
    /// HTML has newlines and spaces added, rather then condensed.
    #[test]
    fn chapter_test_tidy() {
        let html = String::from(
            r#"<div>
        <a href="/bible/59/GEN.2.ESV">
        <div class="flex items-center justify-center bg-white z-[5] h-[50px] w-[50px] rounded-full border-gray-15 border-small border-solid shadow-light-1 active:shadow-dark-1 active:bg-gray-5 active:border-b-gray-5">
        <svg width="25" height="25" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg" aria-labelledby="Next Chapter" class="text-gray-25">
        <title id="Next Chapter">Next Chapter</title>
        <path fill-rule="evenodd" clip-rule="evenodd" d="M8.293 18.707a1 1 0 0 1 0-1.414l4.94-4.94a.5.5 0 0 0 0-.707l-4.94-4.939a1 1 0 0 1 1.414-1.414l5.647 5.646a1.5 1.5 0 0 1 0 2.122l-5.647 5.646a1 1 0 0 1-1.414 0Z" fill="currentColor">
        </path>
        </svg>
        </div>
        </a>
        </div>"#,
        );
        let r = find_next_chapter(&html);
        if !r.is_ok() {
            println!("DEBUG result = {:?}", r);
        }
        assert!(r.is_ok());
        let link = r.unwrap();
        assert_eq!(link, "/bible/59/GEN.2.ESV");
    }

}
